@@ -103,7 +103,7 @@ class SimpleClient:
         The transport is returned as a string and can be one of ``polling``
         and ``websocket``.
         """
-        return self.client.transport if self.client else ''
+        return self.client.transport() if self.client else ''
 
     def emit(self, event, data=None):
         """Emit an event to the server.
@@ -155,6 +155,8 @@ class SimpleClient:
             try:
                 return self.client.call(event, data, namespace=self.namespace,
                                         timeout=timeout)
+            except TimeoutError:
+                raise
             except SocketIOError:
                 pass
 
