@@ -1,5 +1,6 @@
 import asyncio
 import functools
+import json
 from unittest import mock
 
 import pytest
@@ -56,7 +57,148 @@ class TestAsyncPubSubManager:
             {
                 'method': 'emit',
                 'event': 'foo',
-                'data': 'bar',
+                'binary': False,
+                'data': ['bar'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+
+    async def test_emit_binary(self):
+        await self.pm.emit('foo', b'bar')
+        self.pm._publish.assert_awaited_once_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'_placeholder': True, 'num': 0}], 'YmFy'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+        await self.pm.emit('foo', {'foo': b'bar'})
+        self.pm._publish.assert_awaited_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'foo': {'_placeholder': True, 'num': 0}}], 'YmFy'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+
+    async def test_emit_bytearray(self):
+        await self.pm.emit('foo', bytearray(b'bar'))
+        self.pm._publish.assert_awaited_once_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'_placeholder': True, 'num': 0}], 'YmFy'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+        await self.pm.emit('foo', {'foo': bytearray(b'bar')})
+        self.pm._publish.assert_awaited_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'foo': {'_placeholder': True, 'num': 0}}], 'YmFy'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+
+    async def test_emit_list(self):
+        await self.pm.emit('foo', [1, 'two'])
+        self.pm._publish.assert_awaited_once_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': False,
+                'data': [[1, 'two']],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+        await self.pm.emit('foo', [1, b'two', 'three'])
+        self.pm._publish.assert_awaited_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [
+                    [[1, {'_placeholder': True, 'num': 0}, 'three']], 'dHdv',
+                ],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+
+    async def test_emit_no_arguments(self):
+        await self.pm.emit('foo', ())
+        self.pm._publish.assert_awaited_once_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': False,
+                'data': [],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+
+    async def test_emit_multiple_arguments(self):
+        await self.pm.emit('foo', (1, 'two'))
+        self.pm._publish.assert_awaited_once_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': False,
+                'data': [1, 'two'],
+                'namespace': '/',
+                'room': None,
+                'skip_sid': None,
+                'callback': None,
+                'host_id': '123456',
+            }
+        )
+        await self.pm.emit('foo', (1, b'two', 'three'))
+        self.pm._publish.assert_awaited_with(
+            {
+                'method': 'emit',
+                'event': 'foo',
+                'binary': True,
+                'data': [
+                    [1, {'_placeholder': True, 'num': 0}, 'three'], 'dHdv',
+                ],
                 'namespace': '/',
                 'room': None,
                 'skip_sid': None,
@@ -72,7 +214,8 @@ class TestAsyncPubSubManager:
             {
                 'method': 'emit',
                 'event': 'foo',
-                'data': 'bar',
+                'binary': False,
+                'data': ['bar'],
                 'namespace': '/',
                 'room': sid,
                 'skip_sid': None,
@@ -87,7 +230,8 @@ class TestAsyncPubSubManager:
             {
                 'method': 'emit',
                 'event': 'foo',
-                'data': 'bar',
+                'binary': False,
+                'data': ['bar'],
                 'namespace': '/baz',
                 'room': None,
                 'skip_sid': None,
@@ -102,7 +246,8 @@ class TestAsyncPubSubManager:
             {
                 'method': 'emit',
                 'event': 'foo',
-                'data': 'bar',
+                'binary': False,
+                'data': ['bar'],
                 'namespace': '/',
                 'room': 'baz',
                 'skip_sid': None,
@@ -117,7 +262,8 @@ class TestAsyncPubSubManager:
             {
                 'method': 'emit',
                 'event': 'foo',
-                'data': 'bar',
+                'binary': False,
+                'data': ['bar'],
                 'namespace': '/',
                 'room': None,
                 'skip_sid': 'baz',
@@ -135,7 +281,8 @@ class TestAsyncPubSubManager:
                 {
                     'method': 'emit',
                     'event': 'foo',
-                    'data': 'bar',
+                    'binary': False,
+                    'data': ['bar'],
                     'namespace': '/',
                     'room': 'baz',
                     'skip_sid': None,
@@ -227,6 +374,20 @@ class TestAsyncPubSubManager:
         )
 
     async def test_handle_emit(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({'event': 'foo', 'data': ['bar']})
+            super_emit.assert_awaited_once_with(
+                'foo',
+                'bar',
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
+    async def test_handle_legacy_emit(self):
         with mock.patch.object(
             async_manager.AsyncManager, 'emit'
         ) as super_emit:
@@ -240,6 +401,140 @@ class TestAsyncPubSubManager:
                 callback=None,
             )
 
+    async def test_handle_emit_binary(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'_placeholder': True, 'num': 0}], 'YmFy'],
+            })
+            super_emit.assert_awaited_once_with(
+                'foo',
+                b'bar',
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [[{'foo': {'_placeholder': True, 'num': 0}}], 'YmFy'],
+            })
+            super_emit.assert_awaited_with(
+                'foo',
+                {'foo': b'bar'},
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
+    async def test_handle_legacy_emit_binary(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [{'_placeholder': True, 'num': 0}, 'YmFy'],
+            })
+            super_emit.assert_awaited_once_with(
+                'foo',
+                b'bar',
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [{'foo': {'_placeholder': True, 'num': 0}}, 'YmFy'],
+            })
+            super_emit.assert_awaited_with(
+                'foo',
+                {'foo': b'bar'},
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
+    async def test_handle_emit_list(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({'event': 'foo', 'data': [[1, 'two']]})
+            super_emit.assert_awaited_once_with(
+                'foo',
+                [1, 'two'],
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [
+                    [[1, {'_placeholder': True, 'num': 0}, 'three']], 'dHdv'
+                ]
+            })
+            super_emit.assert_awaited_with(
+                'foo',
+                [1, b'two', 'three'],
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
+    async def test_handle_emit_no_arguments(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({'event': 'foo', 'data': []})
+            super_emit.assert_awaited_once_with(
+                'foo',
+                (),
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
+    async def test_handle_emit_multiple_arguments(self):
+        with mock.patch.object(
+            async_manager.AsyncManager, 'emit'
+        ) as super_emit:
+            await self.pm._handle_emit({'event': 'foo', 'data': [1, 'two']})
+            super_emit.assert_awaited_once_with(
+                'foo',
+                (1, 'two'),
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+            await self.pm._handle_emit({
+                'event': 'foo',
+                'binary': True,
+                'data': [
+                    [1, {'_placeholder': True, 'num': 0}, 'three'], 'dHdv'
+                ]
+            })
+            super_emit.assert_awaited_with(
+                'foo',
+                (1, b'two', 'three'),
+                namespace=None,
+                room=None,
+                skip_sid=None,
+                callback=None,
+            )
+
     async def test_handle_emit_with_namespace(self):
         with mock.patch.object(
             async_manager.AsyncManager, 'emit'
@@ -482,22 +777,20 @@ class TestAsyncPubSubManager:
         host_id = self.pm.host_id
 
         async def messages():
-            import pickle
-
             yield {'method': 'emit', 'value': 'foo', 'host_id': 'x'}
             yield {'missing': 'method', 'host_id': 'x'}
             yield '{"method": "callback", "value": "bar", "host_id": "x"}'
             yield {'method': 'disconnect', 'sid': '123', 'namespace': '/foo',
                    'host_id': 'x'}
             yield {'method': 'bogus', 'host_id': 'x'}
-            yield pickle.dumps({'method': 'close_room', 'value': 'baz',
-                                'host_id': 'x'})
+            yield json.dumps({'method': 'close_room', 'value': 'baz',
+                              'host_id': 'x'})
             yield {'method': 'enter_room', 'sid': '123', 'namespace': '/foo',
                    'room': 'room', 'host_id': 'x'}
             yield {'method': 'leave_room', 'sid': '123', 'namespace': '/foo',
                    'room': 'room', 'host_id': 'x'}
             yield 'bad json'
-            yield b'bad pickled'
+            yield b'bad data'
 
             # these should not publish anything on the queue, as they come from
             # the same host
@@ -505,8 +798,8 @@ class TestAsyncPubSubManager:
             yield {'method': 'callback', 'value': 'bar', 'host_id': host_id}
             yield {'method': 'disconnect', 'sid': '123', 'namespace': '/foo',
                    'host_id': host_id}
-            yield pickle.dumps({'method': 'close_room', 'value': 'baz',
-                                'host_id': host_id})
+            yield json.dumps({'method': 'close_room', 'value': 'baz',
+                              'host_id': host_id})
 
         self.pm._listen = messages
         await self.pm._thread()
